module Addons
  module Recipe
    module Instructions
      class Git
        GIT_EXISTS = {
          "run_msg" => "Verifying you're running this recipe in the top level of a Git repository.",
          "type" => InstructionTypes::DIRECTORY_EXISTS,
          "command" => "./.git",
          "error_msg" => "Error: This is not the top level of a git repository. Must exit.",
        }
        GIT_ALL_CODE_COMMITTED = {
          "run_msg" => "Verifying that you have no uncommitted code in Git",
          "type" => InstructionTypes::TERMINAL_COMMAND,
          "command" => "git status",
          "expected_result" => {
            "type" => ResultTypes::DOES_NOT_CONTAIN_STRING,
            "values" => ["Changes not staged for commit", "Changes to be committed"]
          },
          "error_msg" => "Error: You have unstaged files or uncommitted changes in Git. Please commit or stash everything so we can start on a clean slate.",
        }
        GIT_COMMIT_RECIPE = {
          "run_msg" => "Committing changes made by #{Addons::Recipe::RECIPE_NAME_PLACEHOLDER} version: #{Addons::Recipe::RECIPE_VERSION_PLACEHOLDER} AddonList recipe",
          "type" => InstructionTypes::TERMINAL_COMMAND,
          "command" => "git add .; git commit -m \"ADDONS.IO RECIPE  - #{Addons::Recipe::RECIPE_NAME_PLACEHOLDER} version: #{Addons::Recipe::RECIPE_VERSION_PLACEHOLDER}\"",
          "expected_result" => {
            "type" => ResultTypes::NO_EXPECTATION
          },
          "error_msg" => "Error: Git commit for this recipe failed.",
        }
        GIT_RESET_HARD_HEAD = {
          "run_msg" => "There was an error, so we're undoing our changes. If you cannot fix this based on the error message, contact support@addons.io.",
          "type" => InstructionTypes::TERMINAL_COMMAND,
          "command" => "git add .; git reset --hard HEAD",
          "expected_result" => {
            "type" => ResultTypes::DOES_NOT_CONTAIN_STRING,
            "values" => ["HEAD is now at"]
          },
          "error_msg" => "Error: we could not reset our work in Git. We hope we cleaned up this recipe, but can't be sure."
        }
      end # class Git

      class Rails
        RAILS_INITIALIZERS_EXIST = {
          "run_msg" => "Verifying you have a config/initializers directory, per Rails 4",
          "type" => InstructionTypes::DIRECTORY_EXISTS,
          "command" => "./config/initializers",
          "error_msg" => "Error: This does not look like the Rails directory: config/initializers not found"
        }
        RAILS_INITIALIZERS_ADDONS_DIR = {
          "run_msg" => "Creating config/initializers/addons directory",
          "type" => InstructionTypes::DIRECTORY_CREATE,
          "command" => "./config/initializers/addons",
          "error_msg" => "Error: Could not create ./config/initializers/addons directory"
        }
      end # class Rails

      class Bundler
        BUNDLER_INSTALL = {
          "run_msg" => "running bundle install",
          "type" => InstructionTypes::TERMINAL_COMMAND,
          "command" => "bundle install",
          "expected_result" => {
            "type" => ResultTypes::NO_EXPECTATION
          },
          "error_msg" => "Error: You have unstaged files or uncommitted changes in Git. Please commit or stash everything so we can start on a clean slate."
        }
      end # class Bundler

      class Addons
        ADDON_RECIPES_INSTALLED_EXISTS = {
          "run_msg" => "Checking for your ./addon_recipes_installed.yml file",
          "type" => InstructionTypes::TERMINAL_COMMAND,
          "command" => "touch ./addon_recipes_installed.yml",
          "expected_result" => {
            "type" => ResultTypes::NO_EXPECTATION
          },
          "error_msg" => "Error: Problem creating addon_recipes_installed.yml to record your installed recipes"
        }
        RECIPE_HASNT_BEEN_RUN_BEFORE = {
          "run_msg" => "Verifying you have not installed this recipe before",
          "type" => InstructionTypes::TERMINAL_COMMAND,
          "command" => "grep '<RECIPE_HANDLE_PLACEHOLDER>' ./addon_recipes_installed.yml",
          "expected_result" => {
            "type" => ResultTypes::IS_EXACT_STRING,
            "values" => [""]
          },
          "error_msg" => "Error: This recipe (<RECIPE_NAME_PLACEHOLDER>) has been run before. It cannot be run again."
        }
        RECORD_RECIPE_INSTALLED = {
          "run_msg" => "Adding \"<RECIPE_HANDLE_PLACEHOLDER>: '<RECIPE_VERSION_PLACEHOLDER>'\" to ./addon_recipes_installed.yml",
          "type" => InstructionTypes::MODIFY_CODE,
          "command" => {
            "type" => ModifyCodeTypes::ADD_CODE_WITHOUT_TAGS,
            "file" => "./addon_recipes_installed.yml",
            "code" => "<RECIPE_HANDLE_PLACEHOLDER>: '<RECIPE_VERSION_PLACEHOLDER>'",
          },
          "error_msg" => "Error: Could not record recipe installation in ./addon_recipes_installed.yml"
        }
      end

    end # module Instructions

    class Commons
      GIT_READY_RECIPE = [
        # makes sure user has .git
        Addons::Recipe::Instructions::Git::GIT_EXISTS,
        # make sure no uncommitted code
        Addons::Recipe::Instructions::Git::GIT_ALL_CODE_COMMITTED,
      ]

      START_RECIPE = [
        # verify config/initializers directory
        Addons::Recipe::Instructions::Rails::RAILS_INITIALIZERS_EXIST,

        # make sure we're recording recipes that are being run
        Addons::Recipe::Instructions::Addons::ADDON_RECIPES_INSTALLED_EXISTS,
        # make sure this recipe hasn't been run before
        Addons::Recipe::Instructions::Addons::RECIPE_HASNT_BEEN_RUN_BEFORE,

        # create config/initializers/addons directory
        Addons::Recipe::Instructions::Rails::RAILS_INITIALIZERS_ADDONS_DIR,
      ]

      END_RECIPE = [
        # log this recipe installed in addon_recipes_installed.yml
        Addons::Recipe::Instructions::Addons::RECORD_RECIPE_INSTALLED,
        # commit changes in Git
        Addons::Recipe::Instructions::Git::GIT_COMMIT_RECIPE,
      ]
    end

  end # module Recipe
end # module Addons
