static_recipes = [
  {
    name: "Mailgun SMTP",
    handle: "mailgun_smtp",
    revision: "1",
    platform: "Rails 4",
    instructions: [
      # search for existing smtp_settings and comment them out
      {
        run_msg: "Comment exsiting ActionMailer settings throughout the codebase",
        type: InstructionTypes::SEARCH_AND_COMMENT,
        search_queries: ["ActionMailer::Base.delivery_method", "ActionMailer::Base.smtp_settings"],
        comment_code_if: [/ActionMailer::Base.delivery_method\s*=.*$/, /ActionMailer::Base.smtp_settings\s*=\s*{.*}/m],
        error_msg: "Error: Please remove existing ActionMailer::Base.smtp_settings\n(found with: \"grep 'ActionMailer::Base.smtp_settings' -rI --exclude-dir=\.git ./\")"
      },
      # add config/initializers/mailgun.rb
      {
        run_msg: "Adding config/initializers/addons/mailgun_smtp.rb to configure ActiveMailer for Mailgun SMTP",
        type: InstructionTypes::MODIFY_CODE,
        command: {
          type: ModifyCodeTypes::ADD_CODE_REPLACE_FILE,
          file: "config/initializers/addons/mailgun_smtp.rb",
          code: "\
ActionMailer::Base.smtp_settings = {\n\
  :port           => ENV['MAILGUN_SMTP_PORT'],\n\
  :address        => ENV['MAILGUN_SMTP_SERVER'],\n\
  :user_name      => ENV['MAILGUN_SMTP_LOGIN'],\n\
  :password       => ENV['MAILGUN_SMTP_PASSWORD'],\n\
  :domain         => 'example.com',\n\
  :authentication => :plain,\n\
}\n\
ActionMailer::Base.delivery_method = :smtp\n\
  "
        },
        error_msg: "There was a problem adding Addons::Mailgun.send_simple_email",
      },
    ]
  },
  {
    name: "Mailgun HTTP",
    handle: "mailgun_http",
    revision: "1",
    platform: "Rails 4",
    instructions: [
      # install gem rest_client with Gemfile and run from terminal
      {
        run_msg: "Adding gem 'rest_client' to Gemfile",
        type: InstructionTypes::MODIFY_CODE,
        command: {
          type: ModifyCodeTypes::ADD_CODE,
          file: "Gemfile",
          code: "gem 'rest_client'",
          comment_code_if: /rest_client/,
        },
        error_msg: "Error: Could not add gem 'rest_client' to Gemfile"
      },
      Addons::Recipe::Instructions::Bundler::BUNDLER_INSTALL,
      # add Addons::Mailgun.send_simple_email(email) code
      {
        run_msg: "Adding Addons::Mailgun.send_simple_email for sending simple emails",
        type: InstructionTypes::MODIFY_CODE,
        command: {
          type: ModifyCodeTypes::ADD_CODE_REPLACE_FILE,
          file: "config/initializers/addons/mailgun_http.rb",
          code: "\
require 'rest_client'\n\
\n\
module Addons\n\
  class Mailgun\n\
    def self.send_simple_message(email)\n\
      from = email[:from] || email[\"from\"] || email.from\n\
      to = email[:to] || email[\"to\"] || email.to\n\
      subject = email[:subject] || email[\"subject\"] || email.subject\n\
      body = email[:body] || email[\"body\"] || email.body\n\
\n\
      RestClient.post \"https://api:\#{ENV['MAILGUN_API_KEY']}\"\\\n\
      \"@api.mailgun.net/v2/\#{ENV['MAILGUN_SMTP_LOGIN'].match(/@(.*)$/)[1]}/messages\",\n\
      :from => from,\n\
      :to => to,\n\
      :subject => subject,\n\
      :text => body\n\
    end\n\
  end # Mailgun\n\
end # Addons\n\
  "
        },
        error_msg: "There was a problem adding Addons::Mailgun.send_simple_email",
      },
    ]
  }, # end Mailgun HTTP
]