#lib/generators/addons/install_generator.rb
require 'rails/generators'
require 'rails/generators/named_base'

# creates an initializer and runs Addons::Config.init()"
module Addons
  module Generators
    class InstallGenerator < Rails::Generators::Base

      class_option :apiid, type: :string, required: true, :desc => "ADDONS_API_ID"
      class_option :authtoken, type: :string, required: true, :desc => "ADDONS_AUTH_TOKEN"

      def initialize(*args, &block)
        super

        @ADDONS_API_ID = options[:apiid]
        @ADDONS_AUTH_TOKEN = options[:authtoken]

        puts "ADDONS_API_ID: #{@ADDONS_API_ID}"
        puts "ADDONS_AUTH_TOKEN: #{@ADDONS_AUTH_TOKEN}"
      end

      desc "Addons install generator"

      def create_initializer_file
        create_file "config/initializers/addons.rb", "# Addons initializer content\nAddons::Config.init()"
      end

      def add_application_yml_to_gitignore
        # gitignore config/application.yml so we aren't storing sensitive info in .git
        # TODO: replace figaro with figaro-addons and application.yml with addons.yml
        gitignore = Rails.root.join('.gitignore')
        if File.readlines(gitignore).grep(/config\/application.yml/).size == 0
          File.open(gitignore, 'a') do |file|
            file.puts 'config/application.yml'
          end
        end
      end

      def add_figaro_gem
        figaro = Rails.root.join('Gemfile')
        if File.readlines(figaro).grep(/gem 'figaro'/).size == 0
          File.open(figaro, 'a') do |file|
            file.puts ""
            file.puts "gem 'figaro', '~> 0.7.0' # Addons dependency"
          end
        end

        # bundle install
        result = `bundle install`
      end

      def bootstrap_api_id_and_token
        # write config/application.yml to load env vars on startup
        filename = Rails.root.join('config', 'application.yml')

        # NOTE: 'w' overwrites the previous file!
        File.open(filename, 'w') do |file|
          # add new vars to application.yml for the restart
          file.puts "ADDONS_API_ID: #{@ADDONS_API_ID}"
          file.puts "ADDONS_AUTH_TOKEN: #{@ADDONS_AUTH_TOKEN}"
        end

        puts "created application.yml"
      end

    end
  end
end
